<?php
/**
 * api: php
 * title: config.ini editing
 * description: modifies a .ini file instead of config.php
 * priority: never
 * category: library
 * version: 0.1
 * depends: libconfigedit
 *  
 *  This plugin enables the editing of a config.ini file instead
 *  of the default config.php script.
 *
 *  You have to enable this feature manually however. It's not
 *  used in libconfig normally, and therefore requires:
 *  - define("CONFIG_FILE", "test.ini"); in admin_ui.php
 *  - include("lib/editconfigini.php") as well
 * 
 *
 *  And again: Using a .ini file does NOT provide any factual
 *  security advantage over the config.php scheme.
 *  See the README.
 *
 *  This is just an example on how to add new backends. Unlike
 *  PEAR::Config, the libconfig API is in no way prepared for
 *  that however. It's simple enough here. But anyhow the design
 *  of line-wise editing doesn't lend itself to modifying XML
 *  files. And for SQL config storages libconfig is likewise
 *  overkill, since retaining comments isn't applicable there.
 *
 *
 *  This variant does not respect any [sections] of .ini files,
 *  and still adds # as default comment character. Additionally
 *  it depends on variable names still being arrays: arr[name].
 *  Also it deviates from old type ini files, in that multiple
 *  load= lines can occour for plugins. -> PHPs parse_ini() only
 *  sees the first. So you need an advanced ini parser for the
 *  generated config files.
 *
 *     +-----------------------------------------------------------
 *     |[application]
 *     |
 *   Y |cfg_array[set1] = value
 *   Y |CONSTANT = 123
 *   Y |load = .../plugin.php
 *     |
 *     |; this gets recognized as currently disabled plugin:
 *   C |;load = old.php
 *     |
 *  Note, that in an .ini file, comments cannot appear behind a
 *  variable assignment or load_plugin= line.
 *
 */



class generic_config_edit_ini extends generic_config_edit {



   /**
    * create new include/name/value command
    */
   function new_config_command($ctype, $name, $value) {
      // encode strings for syntactic reasons,
      // but doesn't prevent specials like "? >"
      $value = addslashes($value);
      $name = addslashes($name);  // redundant for name, but do it anyhow
      switch ($ctype) {
         // plugins are easy:
         case 1:
            $s = "load = $value";
            break;
         // variable assignments have Array[args] without the ["quotes"] normally:
         case 2:
            $s = $this->varname_to_php_expression($name)
               . " = ". $this->value_to_php_expression($value) . "";
            break;
         // constants name must be all-alphanumeric
         case 3:
            if (!preg_match("/^\w+$/", $name)) {
               $this->error("constant name '$name' invalid");
            }
            $s = '' . $name . ' = ' . $this->value_to_php_expression($value) . '';
            break;
         default:
      }
      return $s;
   }



   /**
    * stub here
    */
   function varname_to_php_expression($name) {
      return $name;
   }



   /**
    * encapsulate "strings" or convert numbers into integers
    */
   function value_to_php_expression($value) {   
      $q = '';
      return (preg_match("/^\d+$/", $value) ? (int)$value : $q . $this->clean_value($value) . $q);
   }



   /**
    * filter variable/constant value (remove syntax breaking newlines, e.g.)
    */
   function clean_value($v) {
      $v = preg_replace("/[\r\n\001-\007\011-\037\'\"\\\$]/", " ", $v);
      $v= strtr($v, "\000", " ");
      return $v;
   }



   /**
    * categorise each individual line, extract basic info
    * 
    */
   function analyse($lines) {
      $cf = array();
   
      #-- peek at each text line
      #   (- note that we don't trim the right side \n off !)
      foreach ($lines as $line) {
         list($ctype, $name, $value, $sort, $orig, $trail) = array(0, false, false, 0, $line, $line);

         // plugin include
         if (preg_match("<^[#;]*\@?(load|load_?plugin|device|plugin|load_?module|include|require|include_once|require_once)\s*=\s*(\w[-/.\w]+)>ims", $line, $uu)) {
            $ctype = 1;
            $value = $uu[2];
            $name = basename($value, ".php");   // we don't work on .inc files
            $sort = 0;
         }
         // var=value assignment
         elseif (preg_match("<^[#;]*[\$]?([_\w]+\[.+?\])\s*=\s*(?:[\"\'](.*?)[\"\']|([^\"']+))>ims", $line, $uu)) {
            $ctype = 2;
            $name = $this->clean_varname($uu[1]);
            $value = stripslashes($uu[2]);
            if (isset($uu[3])) {
               $value = $uu[3];
            }
            $sort = -100;
         }
         // constant definition
         elseif (preg_match("<^[#;]*?\s*([_\w]+)\s*=\s*[\"\']?(.+?)[\"\']?\)>ims", $line, $uu)) {
            $ctype = 3;
            $name = $uu[1];
            $value = stripslashes($uu[2]);
            $sort = -100;
         }
         // something else
         else {
            # then we don't care
         }
         
         #-- keep cutted-out textline (for later ->add/remove/modify() use)
         if ($ctype && strlen($uu[0])) {
            $trail = substr($orig, strlen($uu[0]));
         }
         
         #-- detect if just commented out (disabled plugin, assign, define)
         if (preg_match("<^[#;]+[@]?[\$]?[\w]+>ims", $line)) {
            $ctype = $ctype * -1;
         }
         
         #-- append line info
         $cf[] = new generic_config_line($ctype, $name, $value, $sort, $orig, $trail);
      }
      
      return($cf);
   }

   

}







?>