<?php
/*
   api: php
   type: class
   title: Unzip
   description: Reads and extracts PKZIP archives in-memory
   category: library
   priority: auto
   author: Flocke, milky
   license: MIT
   version: 1.3
   
   Flockes "Archive on the Fly"
   A class to extract the contents of ZIP files.

   Edited version 1.2c - always find the latest version at
   http://flocke.vssd.de/prog/code/php/aotf/

   Copyright (C) 2005 Volker Siebert <flocke@vssd.de>
   All rights reserved.

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
   DEALINGS IN THE SOFTWARE.
*/
 

class vsUnzipFile {


   var $handle;     // Filehandle to opened zip
   var $filesize;   // Size of archive
   var $names;      // List with intindex=>names
   var $directory;  // Central directory
   var $count;      // Number of entries in central directory
   var $comment;    // ZIP file comment
   var $errors;     // Array with error messages



   // Constructor
   function vsUnzipFile()
   {
      $this->handle = 0;
      $this->errors = array();
      $this->close();
   }



   // Close archive
   function close()
   {
      if (!empty($this->handle)) {
         fclose($this->handle);
      }
      $this->handle = 0;
      $this->filesize = 0;
      $this->count = 0;
      $this->names = array();
      $this->directory = array();
   }



   // dosdate-dword to unix timestamp
   function _dostime2time($dostime)
   {
      return mktime(
         ($dostime >> 11) & 0x1f,
         ($dostime >>  5) & 0x3f,
         ($dostime <<  1) & 0x3f,
         ($dostime >> 21) & 0x0f,
         ($dostime >> 16) & 0x1f,
        (($dostime >> 25) & 0x7f) + 1980
      );
   }



   // decode "Central Directory File Header"
   var $CDFH = 'Vmagic/v2version/vgpbf/vmethod/Vdosdate/Vcrc32/Vcsize/Vosize/vnamelen/vextra/vcmntlen/vdisk/viattr/Veattr/Voffset';
   function cdfh($dir, $offs)
   {
      // read
      $hdr = unpack($this->CDFH, $dir);

      // check    # PK^A^B...
      if ($this->valid_header($hdr, 0x02014b50, $offs, "CDFH")) {

         // cut data
         $this->patch_fields($hdr, $dir, $nameoffset=46);
         return($hdr);
      }
   }



   // decode "Local File Header"
   var $LFH = 'Vmagic/vversion2/vgpbf/vmethod/Vdosdate/Vcrc32/Vcsize/Vosize/vnamelen/vextra';
   function lfh($dir)
   {
      // read
      $hdr = unpack($this->LFH, $dir);

      // check   # PK^C^D...
      if ($this->valid_header($hdr, 0x04034b50, 31, "LFH")) {

         // return data
         $this->patch_fields($hdr, $dir, $nameoffset=30);
         return($hdr);
      }
   }
   
   

   // test magic value and compression method, offsets, ... for CDFH and LFH
   function valid_header($hdr, $magic, $offs=0, $title)
   {
      // magic no
      if (($magic != $hdr["magic"]) or ($hdr["version2"] > 20)
      or (@$hdr["offset"] < 0) or (@$hdr["offset"] > $offs - 30) )
      {
      print_r($hdr);
         return $this->error("Invalid $title entry");
      }
      
      // compression method
      if ($hdr["method"] & 0xF7) {   // only =0 and =8
         return $this->error("Unknown compression method");
      }
      
      // else all fine
      return TRUE;
   }
   


   // unpack common dir/file header fields
   function patch_fields(&$hdr, $dir, $nameoffset)
   {
      // cut out filename, convert time
      $hdr["name"] = substr($dir, $nameoffset, $hdr["namelen"]);
      $hdr["time"] = $this->_dostime2time($hdr["dosdate"]);
      $hdr["hdrlen"] = $nameoffset + $hdr["namelen"] + $hdr["extra"] + (int)@$hdr["cmntlen"];
#<off>#$hdr["extra"] = substr($dir, $nameoffset + $hdr["namelen"], $hdr["extra"]);

      // file attributes
      $hdr["eattr"] = $hdr["eattr"] & 0x3f;
      if (substr($name, -1, 1) == '/') {
         $hdr["eattr"] = $hdr["eattr"] | 0x10;
      }

      // empty file
      if ($hdr["osize"] == 0) {
         $hdr["data"] = "";
      }
   }



   // read archive in
   var $ZIPH = "PK\005\006\000\000\000\000";
   var $DISKH = 'Vmagic/v2disk/v2count/Vdirlen/Voffset/vcomment';
   function open($filename)
   {
      // init, open file
      $this->vsUnzipFile();
      if ((($this->filesize = filesize($filename)) < 22) or !($this->handle = @fopen($filename, "rb"))) {
         return $this->error("Cannot open input file");
      }

      // read near EOF
      $offs = $this->filesize > 1024 ? $this->filesize - 1024 : 0;
      fseek($this->handle, $offs, SEEK_SET);
      $TAIL = fread($this->handle, 1024);

      // search for PK^E^F^@^@^@^@
      $pos = strpos($TAIL, $this->ZIPH);
      if ($pos === false) {
         return $this->close() or $this->error("No ZIP header found");
      }

      // move pointer to real header
      $TAIL = substr($TAIL, $pos);
      $offs += $pos;

      // check EndOfDirectory header
      $eod = unpack($this->DISKH, $TAIL);
      $cdfh_offs = $eod["offset"];
      $cdfh_len = $eod["dirlen"];
      // verify header (magic value already seen in ->ZIPH)
      if ($eod["count1"]!=$eod["count2"] || $cdfh_len<0 || $cdfh_offs<0 || $cdfh_offs+$cdfh_len>$offs)
      {
         return $this->close() or $this->error("Invalid ZIP header");
      }
      $this->comment = substr($TAIL, 22, $eod["comment"]);
      $TAIL = "";


      // read central directory
      fseek($this->handle, $cdfh_offs, SEEK_SET);
      $dir = fread($this->handle, $cdfh_len);

      // file by file
      while (strlen($dir))
      {
         // read entry
         $cde = $this->cdfh($dir, $cdfh_offs);
         if (!$cde) {
            return $this->close();
         }
         
         // add entry
         $name = $cde["name"];
         $this->count += 1;
         $this->names[] = $name;
         $this->directory[$name] = $cde;

         // move pointer
         $dir = substr($dir, $cde["hdrlen"]);
      }

      return true;
   }



   // convert integer index to filename
   function name_index($index)
   {
      if (is_int($index)) {
         $index = $this->names[intval($index)];
      }

      return "$index";
   }



   // Return an entry from the directory
   function &get_entry($index)
   {
      return $this->directory[$this->name_index($index)];
   }



   // Return the data for the given entry
   function &get_data($index)
   {
      // get from CentralDirectory
      $entry =& $this->get_entry($index);
      if (isset($entry["data"])) {
         return $entry["data"];
      }

      // read LocalDir header      
      fseek($this->handle, $entry["offset"], SEEK_SET);
      $dir = fread($this->handle, 30 + 512);
      $test = $lfh = $this->lfh($dir);

      // compare LFH : CDFH
      unset($test["magic"]);
      unset($test["extra"]);
      unset($test["hdrlen"]);
      unset($test["eattr"]);
      if (!$lfh || array_diff($test, $entry)) {
         $this->error("Central directory / local file header mismatch");
         $entry["error"] = 1;
         return $entry["data"] = "";
      }

      // read file from archive
      fseek($this->handle, $entry["offset"] + $lfh["hdrlen"], SEEK_SET);
      $entry["data"] = fread($this->handle, $entry["csize"]);
      // decompress
      if ($entry["method"] == 8) {
         $entry["data"] = gzinflate($entry["data"], $entry["osize"]);
      }

      // compare checksum
      if (crc32($entry["data"]) != $entry["crc32"]) {
         $entry["error"] = 1;
         return $this->error("Checksum error");
      }

      // ok
      return $entry["data"];
   }
   
   
   
   // store error
   function error($msg) {
      $this->errors[] = $msg;
      trigger_error($msg, E_USER_WARNING);
      return false;
   }

}

?>